/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/


#include <libc.h>
#include <string.h>
#include <stdio.h>

#include "OO7.hxx"
#include "GenParams.h"
#include "BenchParams.h"

//////////////////////////////////////////////////////////////////
//
// Query #5 - find all base assemblies B that "use" a composite
// part with a more recent build date than B's build date.
//
/////////////////////////////////////////////////////////////////

extern int	debugMode;

int PrintBaseId(int id)
{
    printf("PrintBaseId(base id = %d)\n", id);
    return TRUE;
}


int PrintBaseCompIds(int bid, int cid)
{
    printf("PrintBaseCompIds(base id = %d, comp id = %d)\n", bid, cid);
    return TRUE;
}


//////////////////////////////////////////////////////////////////
//
// Routine to do Query #5
//
//////////////////////////////////////////////////////////////////


int query5()
{
    // find the base assemblies that use more recently build-dated
    // composite parts;  uses nested foreach loops since they will be
    // more efficient and the ObjectStore query processor won't have
    // much opportunity to do anything fancy here anyway

    if (debugMode) {
        printf("    In Query5, about to execute the query...\n");
    }

    int count = 0;

    // first find the desired base assemblies
    
    Iterator<BaseAssembly> iterBase(*BaseAssembly::extent) ;
    for (; iterBase.more(); iterBase.next()) {
      BaseAssembly* base = iterBase.get() ;
      Iterator<CompositePart> iterComp(base->componentsPriv) ;
      for (; iterComp.more(); iterComp.next()) {
        CompositePart* comp = iterComp.get() ;
	if (debugMode) {
	  printf("    In Query5, checking base assembly %d, buildDate = %d\n",
		 base->id, base->buildDate);
	  printf("               against composite part %d, buildDate = %d\n",
		 comp->id, comp->buildDate);
	}
	if (comp->buildDate > base->buildDate) {
	  base->DoNothing();
	  count++;
	}
      }
    }
    
    return count;
}


